#include <CGAL/basic.h>
#include <CGAL/Cartesian.h>
#include <CGAL/Point_2.h>
#include <CGAL/Triangulation_euclidean_traits_2.h>
#include <CGAL/Delaunay_triangulation_2.h>
#include <CGAL/point_generators_2.h>
#include <CGAL/random_selection.h>
#include <GL/glut.h>
#include "openGL_helpers.h"

typedef CGAL::Cartesian<double>                      R;
typedef CGAL::Triangulation_euclidean_traits_2<R>    Gt;
typedef CGAL::Triangulation_vertex_base_2<Gt>        Vb;
typedef CGAL::Triangulation_face_base_2<Gt>          Fb;
typedef CGAL::Triangulation_default_data_structure_2<Gt,Vb,Fb>  Tds;
typedef CGAL::Delaunay_triangulation_2<Gt,Tds>       Triangulation;
typedef Gt::Point                                    Point;
typedef CGAL::Creator_uniform_2<double,Point>        Creator;
typedef Triangulation::Face                          Face;
typedef Triangulation::Face_iterator                 Face_iterator;

enum Menu_constants { QUIT = 0 };


Triangulation T;

class DrawTriangle
{
  public:
    void
    operator()( const Face& f) const
    {
      glBegin( GL_LINE_LOOP);
        glVertex3d(f.vertex(0)->point().x(), f.vertex(0)->point().y(), 0.0);
        glVertex3d(f.vertex(1)->point().x(), f.vertex(1)->point().y(), 0.0);
        glVertex3d(f.vertex(2)->point().x(), f.vertex(2)->point().y(), 0.0);
      glEnd();
    }
};


void
delaunay()
{
  glClear( GL_COLOR_BUFFER_BIT );
  glColor3f( 0.4, 0.8, 1.0);
  for_each( T.faces_begin(), T.faces_end(), DrawTriangle());
  glutSwapBuffers();
}


int
main(int argc, char **argv)
{ 
  CGAL::Random_points_in_disc_2<Point,Creator> gen( 0.9);
  std::copy_n( gen, 501, std::back_inserter(T));

  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
  glutInitWindowSize(600,600);
  glutInitWindowPosition(250,250);
  glutCreateWindow("Delaunay Triangulation");
  glutDisplayFunc(delaunay);
  glutReshapeFunc(reshape); 
  glutCreateMenu(menu);
  glutAddMenuEntry("Quit",QUIT);
  glutAttachMenu(GLUT_RIGHT_BUTTON);
  glutMainLoop();

  return EXIT_SUCCESS;
}

