// file: function_object.C

#include <iterator>
#include <cassert>
#include <fstream>
#include <list>
#include <algorithm>

class Wurzelziehen
{
  public:
    // Funktionsaufrufoperator:
    double operator() (const double& d) const { return std::sqrt(d); }
};


using namespace std;

int
main(int argc, char** argv)
{
  Wurzelziehen function_object;

  cout << function_object(144.0) << endl;

  assert( argc == 2);
  ifstream F(argv[1]);
  list<double> L;
  istream_iterator<double>  start(F);
  istream_iterator<double>  ende;
  copy( start, ende, front_inserter(L) );
  copy( L.begin(), L.end(), ostream_iterator<double>( cout, " ^ "));
  cout << endl;

  transform( L.begin(), L.end(), 
             ostream_iterator<double>(cout, " @ "),  // OutputIterator
             Wurzelziehen()                          // UnaryOperation
           );
  cout << endl;

  return 0;
}

// Vorteil von Funktionsobjekten gegenueber Funktionspointern:
// Es gibt zugeordnete Daten.
/*
class LessDistanceToPoint
{
  public:
    LessDistanceToPoint(const Point& p) : c(p) {}     
    bool operator(const Point& a, const Point& b);
         // liefert true, falls Abstand ac kleiner als Abstand bc
  private:
    Point c;
}

Point q;
// ...
sort(points.begin(), points.end(), LessDistanceToPoint(q) );
// ...
*/
