// file: iterator_traits.C

#include <iterator>

// iterator traits geben Auskunft ueber Iterator:
// aus <iterator>
/* 
template <class _Iterator>
struct iterator_traits 
{
  typedef typename _Iterator::iterator_category iterator_category;
  typedef typename _Iterator::value_type        value_type;
  typedef typename _Iterator::difference_type   difference_type;
  typedef typename _Iterator::pointer           pointer;
  typedef typename _Iterator::reference         reference;
};

template <class _Tp>            // partielle Spezialisierung
struct iterator_traits<_Tp*> 
{
  typedef random_access_iterator_tag  iterator_category;
  typedef _Tp                         value_type;
  typedef ptrdiff_t                   difference_type;
  typedef _Tp*                        pointer;
  typedef _Tp&                        reference;
};

template <class _Tp>            // partielle Spezialisierung
struct iterator_traits<const _Tp*> 
{
  typedef random_access_iterator_tag  iterator_category;
  typedef _Tp                         value_type;
  typedef ptrdiff_t                   difference_type;
  typedef const _Tp*                  pointer;
  typedef const _Tp&                  reference;
};

*/

// moegliche Typen bei iterator_category:
// 
// input_iterator_tag
// output_iterator_tag
// forward_iterator_tag       : public input_iterator_tag
// bidirectional_iterator_tag : public forward_iterator_tag
// random_access_iterator_tag : public bidirectional_iterator_tag


#include <cassert>
#include <fstream>
#include <vector>
#include <list>
#include <algorithm>          // sort template

template <typename ForwardIterator>
void 
my_sort(ForwardIterator first, ForwardIterator beyond, 
        input_iterator_tag);

template <typename RandomAccessIterator>
void 
my_sort(RandomAccessIterator first, RandomAccessIterator beyond, 
        random_access_iterator_tag);

template <typename ForwardIterator>
void
my_sort(ForwardIterator first, ForwardIterator beyond)
{
  typedef typename iterator_traits<ForwardIterator>::iterator_category  IterCat;
  my_sort( first, beyond, IterCat() );
}

template <typename ForwardIterator>
void 
my_sort(ForwardIterator first, ForwardIterator beyond, input_iterator_tag)
{
  typedef typename iterator_traits<ForwardIterator>::value_type   Type;
  std::vector<Type>  V;
  std::copy( first, beyond, back_inserter(V) ); 
  std::sort( V.begin(), V.end() );
  std::copy( V.begin(), V.end(), first );
}



using namespace std;

int
main(int argc, char** argv)
{
  assert( argc == 2);
  ifstream F(argv[1]);
  list<int> L;                             // hat BidirectionalIterator
  istream_iterator<int>  start(F);         // InputIterator
  istream_iterator<int>  ende;               
  copy( start, ende, inserter(L, L.begin()) );
  my_sort( L.begin(), L.end());    // besser: L.sort(L.begin(), L.end())
  copy( L.begin(), L.end(), ostream_iterator<int>( cout, " # "));
  cout << endl;

  return 0;
}
